/////////////////////
//
// Feature to Add: Scroll through locations
//
// Option: Periodically show what the location is
//
// 



const char server[] = "api.openweathermap.org";    // name address for openweathermap (using DNS)




#include <SPI.h>
#include <Wire.h>
#include <Adafruit_GFX.h>
#include <Adafruit_SSD1306.h>

//#include <ESP8266WiFi.h>
//#include <ESP8266HTTPClient.h>


#define SCREEN_WIDTH 128 // OLED display width, in pixels
#define SCREEN_HEIGHT 64 // OLED display height, in pixels


// Stores the minimum and maximum temperature for the next 24 hrs
float maxTemp;
float minTemp;

int displayMode = 0; // 0=weather, 1=clock, (2=mystery)
int maxDisplayMode = 2; // set this to the maximum number of modes

#define OLED_RESET 15  // unused pin
Adafruit_SSD1306 display(SCREEN_WIDTH, SCREEN_HEIGHT, &Wire, OLED_RESET);

#define BUTTON_PIN 14 // pin that is used for the button

#include <Fonts/FreeSans9pt7b.h>
#include <Fonts/FreeSans12pt7b.h>
#include <Fonts/FreeSans18pt7b.h>

int buttonPressedDownTime = 0;
bool myButtonDown = false;


void setMyInterrupts() {
  attachInterrupt(digitalPinToInterrupt(BUTTON_PIN), startMessage, FALLING);  // left button
}

void detachMyInterrupts() {
  detachInterrupt(digitalPinToInterrupt(BUTTON_PIN)); // turn off interrupts
  Serial.println("Detaching interrupts");
}

#include "settings.h"
#include "weather_icons_all_64.h"

#include "flappyDuino.h"
#include "clockMode.h"

#include "accessPoint.h"

bool specialMessage = false; // reset to true for the first time through a mode.

bool showSecretMode = false; // set this to false ** for debug - set to true
#include "secretMode.h"



//OpenWeatherMap settings
#include <JsonListener.h>
#include <OpenWeatherMapCurrent.h>
#include <OpenWeatherMapForecast.h>

OpenWeatherMapCurrentData currentWeather;
OpenWeatherMapForecastData forecasts[MAX_FORECASTS];





unsigned long finishTime = 0;

int buttonTimeDelay = 200; // number of milliseconds to debounce the button
unsigned long buttonResetTime = 0;

//  Note display is setup on clockMode.h



// Weather icon setup

//const unsigned char* getMeteoconIconFromProgmem(String iconText);
////  This is modified from "esp8266-weather-station-color" from adafruit.com
/////  gfx.drawPalettedBitmapFromPgm(0, 55, getMeteoconIconFromProgmem(currentWeather.icon));



void buttonMessage() {
  display.setFont(&FreeSans9pt7b);
  display.clearDisplay();
  display.setCursor(0, 30);
  display.println("Merry Christmas,");
  display.print(owner);
  display.println("!");
  display.display();
  delay(3000);
}

void startMessage() {
  // interrupt handler - indicate that the message should be displayed.
  specialMessage = true;
  incrementDisplayMode();
  buttonResetTime = millis() + buttonTimeDelay; // identify the time when interrupts should be turned back on (for debouncing the button).
  detachMyInterrupts();
}

void incrementDisplayMode() {
  displayMode = constrain(((displayMode + 1) % (maxDisplayMode + 1)), 0, maxDisplayMode);
  //Serial.print ("mode: ");
  //Serial.println(displayMode);
  finishTime = 0; // reset weather timer to make sure the weather will re-display
  checkTime = 0; // reset clock timer make sure the time will re-display
}

void startupDisplay() {

  int iconX = -3;
  int iconY = 5;
  int iconXSize = 64;
  int iconYSize = 64;

  // Draw the temperature and icons for weather
  display.clearDisplay();
  display.setFont(&FreeSans12pt7b);
  //display.setCursor(50, 30);
  centerString(owner + "'s", 30);
  //display.print("102");  // current temperature
  display.setFont(&FreeSans12pt7b);
  //display.setCursor(55, 50);
  centerString("STATION",48); // min and max temperature
  display.drawLine(10,49, 128-10, 49, WHITE);
  
  display.display();
  delay(4000);
  display.setFont();
}


void setup() {
  // put your setup code here, to run once:




  Serial.begin(115200);
  delay(10);

  display.begin(SSD1306_SWITCHCAPVCC, 0x3C);  // initialize with the I2C addr 0x3C (for the 128x64)
  //Wire.pins(0, 2); // SDA, SCL
  //Wire.begin();
  display.setTextWrap(true);
  display.setFont();
  //display.setFont(&FreeSans18pt7b);
  display.setTextSize(0);
  display.setTextColor(WHITE);
  Wire.setClock(1000000);
  display.display();
  delay(3000);
  display.clearDisplay();

  //display.setFont(&FreeSans9pt7b);
  display.setFont();
  display.setCursor(0, 30);
  //display.println("Connecting to:");

  //  display.println(WLAN_SSID);
  display.display();
  
  //
  //  Serial.println(); Serial.println();
  //  Serial.print("Connecting to ");
  //  Serial.println(WLAN_SSID);
  //
  //
  //  WiFi.begin(WLAN_SSID, WLAN_PASS);
  //  while (WiFi.status() != WL_CONNECTED) {
  //    delay(500);
  //    Serial.print(".");
  //    display.print(".");
  //    display.display();
  //  }
  // delay(1000);


  initHardware();

  // if this is the first time through, reset the number of clicks to Zero in the EEPROM
  if (firstTimeThrough) {
    numberOfClicks = 0;
    writeNumberOfClicks();
    firstTimeThrough = false;
    Serial.println("Resetting EEPROM clicks to zero");
  }



  // Try and restore saved settings from EEPROM
  if (loadSavedConfig()) {
    //  if (false) { /// For DEBUG only ***  to force into accesspoint mode, replace the above line with this one.
    if (checkWiFiConnection()) {
      SETUP_MODE = false;
    }
    else {
      SETUP_MODE = true;
      setupMode();
      // Turn the status led Green when the WiFi has been connected
      //     digitalWrite(LED_GREEN, HIGH);
      return;
    }
  }




  pinMode(BUTTON_PIN, INPUT_PULLUP);  // initialize buttons for input
  setMyInterrupts();
  
  delay(1000);
  startupDisplay(); //Display the welcome screen

}

void loop() {


  // this checks to see if the button is held down for a long time to trigger SECRET MODE
  if (digitalRead(BUTTON_PIN) == false ) { // if button is down
    if (myButtonDown == false) {
      buttonPressedDownTime = millis(); // the button just got pressed
      myButtonDown = true; // set that the button is now down
    }
    //check if button is being held down for "maxDelayTime"
    if ((millis() - buttonPressedDownTime) >= maxDelayTime) {
      showSecretMode = true;
    }
  } else (myButtonDown = false);  // indicate that the button is now up


  // This is to debounce the button.
  if ((millis() > buttonResetTime) && (displayMode != 2)) {
    setMyInterrupts();  //Turn back on the interrupts.
  }



  if (SETUP_MODE) {
    DNS_SERVER.processNextRequest();
  }
  WEB_SERVER.handleClient();

  if (WiFi.status() == WL_CONNECTED) {

    if (showSecretMode == false) {
      switch (displayMode) {
        case 0: { // Weather mode
            if (specialMessage == true) {
              finishTime = 0; // reset time to check the weather
              specialMessage = false;
            }
            showWeather();
            break;
          }
        case 1: { // Clock mode
            if (specialMessage == true) {
              checkTime = 0; // reset time to check the time
              specialMessage = false;
            }
            showClock();
            break;
          }

        case 2: { // Flappy bird game
            if (specialMessage == true)
            {
              flappySetup();
              specialMessage = false;
            }

            flappyLoop();
            break;
          }

        default: {
            delay(10);
            break;
          }
      }
    } else {
      doSecretMode();  //
    }
  }
  else if (SETUP_MODE == false) {
    SETUP_MODE = true;
    setupMode();
    // Turn the status led Green when the WiFi has been connected
    //     digitalWrite(LED_GREEN, HIGH);
    return;
  }
}




void showWeather() {
  if (millis() > finishTime) {
    // put your main code here, to run repeatedly:
    //    display.clearDisplay();
    display.drawCircle(0, 0, 1, WHITE);  // draw a circle to indicate weather is being checked.
    display.drawCircle(0, 0, 3, WHITE);  // draw a circle to indicate weather is being checked.
    display.drawCircle(0, 0, 5, WHITE);  // draw a circle to indicate weather is being checked.
    display.display();
    delay(100);
    updateWeather();
    finishTime = millis() + 30 * 1000 * 1;
  } else {
    delay(50);
  }
}
void updateWeather() {

  //  if (WiFi.status() != WL_CONNECTED) {
  //    WiFi.printDiag(Serial);
  //    delay(100);
  //    Serial.print("Reconnecting");
  //    WiFi.begin(WLAN_SSID, WLAN_PASS);
  //    while (WiFi.status() != WL_CONNECTED) {
  //      delay(500);
  //      Serial.print(".");
  //    }
  //
  //  }

  currentWeather.weatherId = 0; // reset the current weather

  while (currentWeather.weatherId == 0) {  // this makes sure that a valid weather was received.
    Serial.println("Updating conditions...");
    OpenWeatherMapCurrent *currentWeatherClient = new OpenWeatherMapCurrent();
    currentWeatherClient->setMetric(IS_METRIC);
    currentWeatherClient->setLanguage(OPEN_WEATHER_MAP_LANGUAGE);
    currentWeatherClient->updateCurrentById(&currentWeather, OPEN_WEATHER_MAP_APP_ID, OPEN_WEATHER_MAP_LOCATION_ID);
    delete currentWeatherClient;
    currentWeatherClient = nullptr;

    Serial.println(currentWeather.cityName);
    Serial.println(currentWeather.description);
    Serial.print("T: ");
    Serial.println(currentWeather.temp);
    Serial.print("ID: ");
    Serial.println(currentWeather.weatherId);
    Serial.print("icon: ");
    Serial.println(currentWeather.icon);

  }

  Serial.println("Updating forecasts...");
  OpenWeatherMapForecast *forecastClient = new OpenWeatherMapForecast();
  forecastClient->setMetric(IS_METRIC);
  forecastClient->setLanguage(OPEN_WEATHER_MAP_LANGUAGE);
  uint8_t allowedHours[] = {12, 9, 6, 0};
  forecastClient->setAllowedHours(allowedHours, sizeof(allowedHours));
  forecastClient->updateForecastsById(forecasts, OPEN_WEATHER_MAP_APP_ID, OPEN_WEATHER_MAP_LOCATION_ID, MAX_FORECASTS);
  delete forecastClient;
  forecastClient = nullptr;

  Serial.println();
  Serial.println("Temperature Forecasts");

  // reset the minimum and maximum temperatures for 24 hrs
  maxTemp = currentWeather.temp;
  minTemp = currentWeather.temp;

  for (int i = 0; i < 8; i++) {
    Serial.print(forecasts[i].temp);
    Serial.print(" temp_min: ");
    Serial.print(forecasts[i].tempMin);
    Serial.print(" temp_max: ");
    Serial.println(forecasts[i].tempMax);

    maxTemp = max(maxTemp, forecasts[i].tempMax);
    minTemp = min(minTemp, forecasts[i].tempMin);
  }


  drawMyText();

}

void drawMyText() {

  int iconX = -3;
  int iconY = 5;
  int iconXSize = 64;
  int iconYSize = 64;

  // Draw the temperature and icons for weather
  display.clearDisplay();
  display.setFont(&FreeSans18pt7b);
  display.setCursor(62, 30);
  display.print(String(currentWeather.temp, 0));
  //display.print("102");  // current temperature
  display.setFont(&FreeSans12pt7b);
  display.setCursor(55, 60);
  display.print(String(minTemp, 0) + "-" + String(maxTemp, 0)); // min and max temperature

  /// Draw the icon of the current weather.
  display.drawBitmap(iconX, iconY, getMeteoconIconFromProgmem(currentWeather.icon), iconXSize, iconYSize, WHITE);
  display.display();
}
